#include <windows.h>
#include <stdio.h>
#include "resource.h"
#include "LEPCLIB_270.h"
#define MAX_SERIALNO_LENGTH 256
#define SERIALNO ""
HWND g_hwnd;

BOOL CALLBACK DlgProcMain( HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam );
void CallFunction(WPARAM wParam);
void PutResult(const char *str);
void CBFunc( void *buf, int len );

int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow )
{
	DialogBox( hInstance, MAKEINTRESOURCE( IDD_LEPCLIB_270_SAMPLE ), NULL, DlgProcMain );
	int a = GetLastError();
	return 0;
}

BOOL CALLBACK DlgProcMain( HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	g_hwnd = hwnd;
    switch( uMsg ){
    case WM_INITDIALOG:
		SetWindowText(GetDlgItem(hwnd,IDC_SERIAL), SERIALNO);
        return TRUE;
    case WM_CLOSE:
        EndDialog( hwnd, 0 );
        return TRUE;
    case WM_COMMAND:
		CallFunction(wParam);
        return TRUE;
	}
	return FALSE;
}

void CallFunction(WPARAM wParam)
{
	char tmp[MAX_SERIALNO_LENGTH+1];
	switch (LOWORD(wParam)) {
	case IDC_LEConnect:
		PutResult("LEConnect:");
		GetWindowText(GetDlgItem(g_hwnd,IDC_SERIAL),tmp,MAX_SERIALNO_LENGTH);
		if( LEConnect(tmp) == LE_SUCCESS ){
			PutResult("SUCCESS\r\n");
		} else {
			PutResult("FAIL\r\n");
		}
		return;
	case IDC_LEDisconnect:
		PutResult("LEDisconnect:");
		if( LEDisconnect() == LE_SUCCESS ){
			PutResult("SUCCESS\r\n");
		} else {
			PutResult("FAIL\r\n");
		}
		return;
	case IDC_LEStartOnlineCallBack:
		PutResult("LEReStartCallBack:");
		if( LESetInterface(1,LE_INTERFACETYPE_LIN) != LE_SUCCESS ){
			PutResult("FAIL\r\n");
			return;
		}
		if( LESetInterface(2,LE_INTERFACETYPE_LIN) != LE_SUCCESS ){
			PutResult("FAIL\r\n");
			return;
		}

		if( LEStartOnlineCallBack(CBFunc) == LE_SUCCESS ){
			PutResult("SUCCESS\r\n");
		} else {
			PutResult("FAIL\r\n");
		}
        return;
	case IDC_LEStop:
		PutResult("LEStop:");
		if( LEStop() == LE_SUCCESS ){
			PutResult("SUCCESS\r\n");
		} else {
			PutResult("FAIL\r\n");
		}

        return;
	default:
        return;
	}
}

void PutResult(const char *str)
{
	SendMessage( GetDlgItem(g_hwnd,IDC_RESULT), EM_REPLACESEL, 0, (LPARAM)str );
}

void CBFunc( void *buf, int len )
{
	LE_MONDATAFORMATCANLIN *data = (LE_MONDATAFORMATCANLIN *)buf;

	int i = 0;
	while(len > 0){
		char str[65536];
		sprintf(str,"%02d:%02d:%02d.%03d.%03d ",
			data[i].TimeHour,
			data[i].TimeMin,
			data[i].TimeSec,
			data[i].TimeMilliSec,
			data[i].TimeMicroSec
			);
		PutResult(str);	
		if( data[i].Channel <= 2 ){
			sprintf(str,"ch:%d CANid:%08x ",
				data[i].Channel,
				data[i].CanId );
			PutResult(str);	
		} else {
			sprintf(str,"ch:%d LINid:%02x %02x ",
				data[i].Channel-2,
				data[i].LinId[0],
				data[i].LinId[1] );
			PutResult(str);	
		}
		int j;
		for( j = 0; j < 8; j ++ ){
			if( j < data[i].DataLength ){
				sprintf(str,"%02x ", data[i].Data[j]);
			} else {
				sprintf(str,"   ");
			}
			PutResult(str);	
		}

		for( j = 0; j < 4; j ++ ){
			float tmp = data[i].AnalogIn[j];
			sprintf(str,"%5.1f ", tmp);
			PutResult(str);	
		}
		switch( data[i].GPS ){
		case LE_MONDATA_NOGPS:	sprintf(str,"NO GPS ");	break;
		default:
		case LE_MONDATA_GPS:	sprintf(str,"   GPS ");	break;
		}
		PutResult(str);	

		if( data[i].GPS != LE_MONDATA_NOGPS ){
			sprintf(str,"sat:%d %02d/%02d/%02d %02d:%02d:%02d ",
				data[i].GPSSatellite,
				data[i].GPSUTCYear,
				data[i].GPSUTCMonth,
				data[i].GPSUTCDate,
				data[i].GPSUTCHour,
				data[i].GPSUTCMinute,
				data[i].GPSUTCSecond
				);
			PutResult(str);	

			sprintf(str,"%s %d %2.4f %s %d %2.4f %d %d ",
				(data[i].GPSLatitudeDir == LE_MONDATA_NORTH) ? "N" : "S",
				data[i].GPSLatitudeDeg,
				data[i].GPSLatitudeSec,
				(data[i].GPSLongitudeDir == LE_MONDATA_EAST) ? "E" : "W",
				data[i].GPSLongitudeDeg,
				data[i].GPSLongitudeSec,
				data[i].GPSAltitude,
				data[i].GPSGeoid
				);
			PutResult(str);	
		}
		sprintf(str,"\r\n");
		PutResult(str);	

		len -= sizeof(LE_MONDATAFORMATCANLIN);
		i++;
	} 
}
